@@ -376,7 +376,7 @@ struct Inner {
     mapped_pipe_transports:
         Arc<Mutex<HashedMap<RouterId, Arc<AsyncMutex<Option<WeakPipeTransportPair>>>>>>,
     // Make sure worker is not dropped until this router is not dropped
-    worker: Option<Worker>,
+    worker: Worker,
     closed: AtomicBool,
     _on_worker_close_handler: Mutex<HandlerId>,
 }
@@ -461,7 +461,8 @@ impl Router {
             let inner_weak = Arc::clone(&inner_weak);
 
             move || {
-                if let Some(inner) = inner_weak.lock().as_ref().and_then(Weak::upgrade) {
+                let maybe_inner = inner_weak.lock().as_ref().and_then(Weak::upgrade);
+                if let Some(inner) = maybe_inner {
                     inner.handlers.worker_close.call_simple();
                     if !inner.closed.swap(true, Ordering::SeqCst) {
                         inner.handlers.close.call_simple();
@@ -480,7 +481,7 @@ impl Router {
             data_producers,
             mapped_pipe_transports,
             app_data,
-            worker: Some(worker),
+            worker,
             closed: AtomicBool::new(false),
             _on_worker_close_handler: Mutex::new(on_worker_close_handler),
         });
@@ -496,6 +497,11 @@ impl Router {
         self.inner.id
     }
 
+    /// Worker to which router belongs.
+    pub fn worker(&self) -> &Worker {
+        &self.inner.worker
+    }
+
     /// Custom application data.
     #[must_use]
     pub fn app_data(&self) -> &AppData {
@@ -1355,17 +1361,16 @@ impl Router {
         // `PipeTransport`. To prevent that, we have `HashedMap` with mapping behind `Mutex` and
         // another `Mutex` on the pair of `PipeTransport`.
 
-        let mut mapped_pipe_transports = self.inner.mapped_pipe_transports.lock();
-
-        let pipe_transport_pair_mutex = mapped_pipe_transports
+        let pipe_transport_pair_mutex = self
+            .inner
+            .mapped_pipe_transports
+            .lock()
             .entry(pipe_to_router_options.router.id())
             .or_default()
             .clone();
 
         let mut pipe_transport_pair_guard = pipe_transport_pair_mutex.lock().await;
 
-        drop(mapped_pipe_transports);
-
         let pipe_transport_pair = match pipe_transport_pair_guard
             .as_ref()
             .and_then(WeakPipeTransportPair::upgrade)
