@@ -164,12 +164,16 @@ impl fmt::Debug for ActiveSpeakerObserver {
     }
 }
 
-#[async_trait(?Send)]
+#[async_trait]
 impl RtpObserver for ActiveSpeakerObserver {
     fn id(&self) -> RtpObserverId {
         self.inner.id
     }
 
+    fn router(&self) -> &Router {
+        &self.inner.router
+    }
+
     fn paused(&self) -> bool {
         self.inner.paused.load(Ordering::SeqCst)
     }
@@ -348,7 +352,8 @@ impl ActiveSpeakerObserver {
             let inner_weak = Arc::clone(&inner_weak);
 
             move || {
-                if let Some(inner) = inner_weak.lock().as_ref().and_then(Weak::upgrade) {
+                let maybe_inner = inner_weak.lock().as_ref().and_then(Weak::upgrade);
+                if let Some(inner) = maybe_inner {
                     inner.handlers.router_close.call_simple();
                     inner.close(false);
                 }
