@@ -139,7 +139,7 @@ struct Inner {
     payload_channel: PayloadChannel,
     handlers: Arc<Handlers>,
     app_data: AppData,
-    transport: Box<dyn Transport>,
+    transport: Arc<dyn Transport>,
     closed: AtomicBool,
     _on_transport_close_handler: Mutex<HandlerId>,
 }
@@ -163,7 +163,7 @@ impl Inner {
                 let channel = self.channel.clone();
                 let request = DataProducerCloseRequest {
                     internal: DataProducerInternal {
-                        router_id: self.transport.router_id(),
+                        router_id: self.transport.router().id(),
                         transport_id: self.transport.id(),
                         data_producer_id: self.id,
                     },
@@ -273,7 +273,7 @@ impl DataProducer {
         channel: Channel,
         payload_channel: PayloadChannel,
         app_data: AppData,
-        transport: Box<dyn Transport>,
+        transport: Arc<dyn Transport>,
         direct: bool,
     ) -> Self {
         debug!("new()");
@@ -285,7 +285,8 @@ impl DataProducer {
             let inner_weak = Arc::clone(&inner_weak);
 
             Box::new(move || {
-                if let Some(inner) = inner_weak.lock().as_ref().and_then(Weak::upgrade) {
+                let maybe_inner = inner_weak.lock().as_ref().and_then(Weak::upgrade);
+                if let Some(inner) = maybe_inner {
                     inner.handlers.transport_close.call_simple();
                     inner.close(false);
                 }
@@ -323,6 +324,11 @@ impl DataProducer {
         self.inner().id
     }
 
+    /// Transport to which data producer belongs.
+    pub fn transport(&self) -> &Arc<dyn Transport> {
+        &self.inner().transport
+    }
+
     /// The type of the data producer.
     #[must_use]
     pub fn r#type(&self) -> DataProducerType {
@@ -429,7 +435,7 @@ impl DataProducer {
 
     fn get_internal(&self) -> DataProducerInternal {
         DataProducerInternal {
-            router_id: self.inner().transport.router_id(),
+            router_id: self.inner().transport.router().id(),
             transport_id: self.inner().transport.id(),
             data_producer_id: self.inner().id,
         }
@@ -444,7 +450,7 @@ impl DirectDataProducer {
         self.inner.payload_channel.notify(
             DataProducerSendNotification {
                 internal: DataProducerInternal {
-                    router_id: self.inner.transport.router_id(),
+                    router_id: self.inner.transport.router().id(),
                     transport_id: self.inner.transport.id(),
                     data_producer_id: self.inner.id,
                 },
