@@ -41,27 +41,6 @@ uuid_based_wrapper_type!(
     TransportId
 );
 
-// We don't want this to be a public API, but have to use it like this to be able to still use as
-// trait object
-mod private {
-    use super::Transport;
-
-    #[doc(hidden)]
-    pub trait CloneTransport {
-        #[doc(hidden)]
-        fn clone_box(&self) -> Box<dyn Transport>;
-    }
-
-    impl<T> CloneTransport for T
-    where
-        T: Transport + Clone + 'static,
-    {
-        fn clone_box(&self) -> Box<dyn Transport> {
-            Box::new(self.clone())
-        }
-    }
-}
-
 /// Data contained in transport trace events.
 ///
 /// See also "trace" event in the [Debugging](https://mediasoup.org/documentation/v3/mediasoup/debugging#trace-Event)
@@ -150,14 +129,20 @@ pub(super) enum TransportType {
 ///
 /// For additional methods see [`TransportGeneric`].
 #[async_trait]
-pub trait Transport: Debug + Send + Sync + private::CloneTransport {
+pub trait Transport: Debug + Send + Sync {
     /// Transport id.
     #[must_use]
     fn id(&self) -> TransportId;
 
     /// Router id.
     #[must_use]
-    fn router_id(&self) -> RouterId;
+    #[deprecated = "Use `router().id()` instead"]
+    fn router_id(&self) -> RouterId {
+        self.router().id()
+    }
+
+    /// Router to which transport belongs.
+    fn router(&self) -> &Router;
 
     /// Custom application data.
     #[must_use]
@@ -275,12 +260,6 @@ pub trait Transport: Debug + Send + Sync + private::CloneTransport {
     fn on_close(&self, callback: Box<dyn FnOnce() + Send + 'static>) -> HandlerId;
 }
 
-impl Clone for Box<dyn Transport> {
-    fn clone(&self) -> Self {
-        self.clone_box()
-    }
-}
-
 /// Generic transport trait with methods available on all transports in addition to [`Transport`].
 #[async_trait]
 pub trait TransportGeneric: Transport + Clone + 'static {
@@ -362,8 +341,6 @@ pub enum ConsumeDataError {
 
 #[async_trait]
 pub(super) trait TransportImpl: TransportGeneric {
-    fn router(&self) -> &Router;
-
     fn channel(&self) -> &Channel;
 
     fn payload_channel(&self) -> &PayloadChannel;
@@ -553,7 +530,7 @@ pub(super) trait TransportImpl: TransportGeneric {
             self.channel().clone(),
             self.payload_channel().clone(),
             app_data,
-            Box::new(self.clone()),
+            Arc::new(self.clone()),
             transport_type == TransportType::Direct,
         );
 
@@ -657,7 +634,7 @@ pub(super) trait TransportImpl: TransportGeneric {
             response.score,
             response.preferred_layers,
             app_data,
-            Box::new(self.clone()),
+            Arc::new(self.clone()),
         ))
     }
 
@@ -728,7 +705,7 @@ pub(super) trait TransportImpl: TransportGeneric {
             self.channel().clone(),
             self.payload_channel().clone(),
             app_data,
-            Box::new(self.clone()),
+            Arc::new(self.clone()),
             transport_type == TransportType::Direct,
         ))
     }
@@ -818,7 +795,7 @@ pub(super) trait TransportImpl: TransportGeneric {
             self.channel().clone(),
             self.payload_channel().clone(),
             app_data,
-            Box::new(self.clone()),
+            Arc::new(self.clone()),
             transport_type == TransportType::Direct,
         );
 
